@@ -46,6 +46,10 @@
 
 #include "svncommitdialog.h"
 #include "svnlogdialog.h"
+#include "svncheckoutdialog.h"
+#include "svnprogressdialog.h"
+#include "svncleanupdialog.h"
+
 #include "svncommands.h"
 
 K_PLUGIN_FACTORY(FileViewSvnPluginFactory, registerPlugin<FileViewSvnPlugin>();)
@@ -61,6 +65,8 @@ FileViewSvnPlugin::FileViewSvnPlugin(QObject* parent, const QList<QVariant>& arg
     m_removeAction(nullptr),
     m_showUpdatesAction(nullptr),
     m_logAction(nullptr),
+    m_checkoutAction(nullptr),
+    m_cleanupAction(nullptr),
     m_command(),
     m_arguments(),
     m_errorMsg(),
@@ -118,10 +124,20 @@ FileViewSvnPlugin::FileViewSvnPlugin(QObject* parent, const QList<QVariant>& arg
             m_showUpdatesAction, &QAction::setChecked);
 
     m_logAction = new QAction(this);
-    m_logAction->setText(xi18nc("@action:inmenu", "SVN Log..."));
+    m_logAction->setText(i18nc("@action:inmenu", "SVN Log..."));
     connect(m_logAction, &QAction::triggered,
             this, &FileViewSvnPlugin::logDialog);
 
+    m_checkoutAction = new QAction(this);
+    m_checkoutAction->setText(i18nc("@action:inmenu", "SVN Checkout..."));
+    connect(m_checkoutAction, &QAction::triggered,
+            this, &FileViewSvnPlugin::checkoutDialog);
+
+    m_cleanupAction = new QAction(this);
+    m_cleanupAction->setText(i18nc("@action:inmenu", "SVN Cleanup..."));
+    connect(m_cleanupAction, &QAction::triggered,
+            this, &FileViewSvnPlugin::cleanupDialog);
+
     connect(&m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
             this, &FileViewSvnPlugin::slotOperationCompleted);
     connect(&m_process, &QProcess::errorOccurred,
@@ -146,7 +162,9 @@ bool FileViewSvnPlugin::beginRetrieval(const QString& directory)
     QMutableHashIterator<QString, ItemVersion> it(m_versionInfoHash);
     while (it.hasNext()) {
         it.next();
-        if (it.key().startsWith(directory)) {
+        // 'svn status' return dirs without trailing slash, so without it we can't remove current
+        // directory from hash.
+        if ((it.key() + QLatin1Char('/')).startsWith(directory)) {
             it.remove();
         }
     }
@@ -257,7 +275,7 @@ KVersionControlPlugin::ItemVersion FileViewSvnPlugin::itemVersion(const KFileIte
     return NormalVersion;
 }
 
-QList<QAction*> FileViewSvnPlugin::actions(const KFileItemList& items) const
+QList<QAction*> FileViewSvnPlugin::versionControlActions(const KFileItemList& items) const
 {
     // Special case: if any item is in unversioned directory we shouldn't add any actions because
     // we can do nothing with this item.
@@ -322,9 +340,23 @@ QList<QAction*> FileViewSvnPlugin::actions(const KFileItemList& items) const
     return actions;
 }
 
+QList<QAction*> FileViewSvnPlugin::outOfVersionControlActions(const KFileItemList& items) const
+{
+    // Only for a single directory.
+    if (items.count() != 1 || !items.first().isDir()) {
+        return {};
+    }
+
+    m_contextDir = items.first().localPath();
+
+    return QList<QAction*>{} << m_checkoutAction;
+}
 
 void FileViewSvnPlugin::updateFiles()
 {
+    SvnProgressDialog *progressDialog = new SvnProgressDialog(i18nc("@title:window", "SVN Update"), m_contextDir);
+    progressDialog->connectToProcess(&m_process);
+
     execSvnCommand(QLatin1String("update"), QStringList(),
                    i18nc("@info:status", "Updating SVN repository..."),
                    i18nc("@info:status", "Update of SVN repository failed."),
@@ -416,13 +448,24 @@ void FileViewSvnPlugin::removeFiles()
 
 void FileViewSvnPlugin::revertFiles()
 {
+    if (m_contextDir.isEmpty() && m_contextItems.empty()) {
+        return;
+    }
+
     QStringList arguments;
+    QString root;
 
     // If we are reverting a directory let's revert everything in it.
     if (!m_contextDir.isEmpty()) {
         arguments << QLatin1String("--depth") << QLatin1String("infinity");
+        root = m_contextDir;
+    } else {
+        root = SvnCommands::localRoot( m_contextItems.last().localPath() );
     }
 
+    SvnProgressDialog *progressDialog = new SvnProgressDialog(i18nc("@title:window", "SVN Revert"), root);
+    progressDialog->connectToProcess(&m_process);
+
     execSvnCommand(QStringLiteral("revert"), arguments,
                    i18nc("@info:status", "Reverting files from SVN repository..."),
                    i18nc("@info:status", "Reverting of files from SVN repository failed."),
@@ -442,6 +485,26 @@ void FileViewSvnPlugin::logDialog()
     svnLogDialog->show();
 }
 
+void FileViewSvnPlugin::checkoutDialog()
+{
+    SvnCheckoutDialog *svnCheckoutDialog = new SvnCheckoutDialog(m_contextDir);
+
+    connect(svnCheckoutDialog, &SvnCheckoutDialog::infoMessage, this, &FileViewSvnPlugin::infoMessage);
+    connect(svnCheckoutDialog, &SvnCheckoutDialog::errorMessage, this, &FileViewSvnPlugin::errorMessage);
+    connect(svnCheckoutDialog, &SvnCheckoutDialog::operationCompletedMessage, this, &FileViewSvnPlugin::operationCompletedMessage);
+
+    svnCheckoutDialog->setAttribute(Qt::WA_DeleteOnClose);
+    svnCheckoutDialog->show();
+}
+
+void FileViewSvnPlugin::cleanupDialog()
+{
+    SvnCleanupDialog *svnCleanupDialog = new SvnCleanupDialog(m_contextDir);
+
+    connect(svnCleanupDialog, &SvnCleanupDialog::errorMessage, this, &FileViewSvnPlugin::errorMessage);
+    connect(svnCleanupDialog, &SvnCleanupDialog::operationCompletedMessage, this, &FileViewSvnPlugin::operationCompletedMessage);
+}
+
 void FileViewSvnPlugin::slotOperationCompleted(int exitCode, QProcess::ExitStatus exitStatus)
 {
     m_pendingOperation = false;
@@ -477,11 +540,18 @@ void FileViewSvnPlugin::slotShowUpdatesToggled(bool checked)
 
 void FileViewSvnPlugin::revertFiles(const QStringList& filesPath)
 {
+    if (filesPath.empty()) {
+        return;
+    }
+
     for (const auto &i : qAsConst(filesPath)) {
         m_contextItems.append( QUrl::fromLocalFile(i) );
     }
     m_contextDir.clear();
 
+    SvnProgressDialog *progressDialog = new SvnProgressDialog(i18nc("@title:window", "SVN Revert"), SvnCommands::localRoot(filesPath.first()));
+    progressDialog->connectToProcess(&m_process);
+
     execSvnCommand(QLatin1String("revert"), QStringList() << filesPath,
                    i18nc("@info:status", "Reverting changes to file..."),
                    i18nc("@info:status", "Revert file failed."),
@@ -562,6 +632,10 @@ void FileViewSvnPlugin::addFiles(const QStringList& filesPath)
 
 void FileViewSvnPlugin::commitFiles(const QStringList& context, const QString& msg)
 {
+    if (context.empty()) {
+        return;
+    }
+
     // Write the commit description into a temporary file, so
     // that it can be read by the command "svn commit -F". The temporary
     // file must stay alive until slotOperationCompleted() is invoked and will
@@ -585,6 +659,9 @@ void FileViewSvnPlugin::commitFiles(const QStringList& context, const QString& m
     m_contextDir.clear();
     m_contextItems.clear();
 
+    SvnProgressDialog *progressDialog = new SvnProgressDialog(i18nc("@title:window", "SVN Commit"), SvnCommands::localRoot(context.first()));
+    progressDialog->connectToProcess(&m_process);
+
     execSvnCommand(QLatin1String("commit"), arguments,
                    i18nc("@info:status", "Committing SVN changes..."),
                    i18nc("@info:status", "Commit of SVN changes failed."),
@@ -666,6 +743,7 @@ QList<QAction*> FileViewSvnPlugin::directoryActions(const KFileItem& directory)
     actions.append(m_removeAction);
     actions.append(m_revertAction);
     actions.append(m_logAction);
+    actions.append(m_cleanupAction);
     return actions;
 }
 
