@@ -190,6 +190,35 @@ QString SvnCommands::remoteRelativeUrl(const QString& filePath)
     }
 }
 
+QString SvnCommands::localRoot(const QString& filePath)
+{
+    QProcess process;
+
+    process.start(
+        QLatin1String("svn"),
+        QStringList {
+            QStringLiteral("info"),
+            QStringLiteral("--show-item"),
+            QStringLiteral("wc-root"),
+            filePath
+        }
+    );
+
+    if (!process.waitForFinished() || process.exitCode() != 0) {
+        return 0;
+    }
+
+    QTextStream stream(&process);
+    QString wcroot;
+    stream >> wcroot;
+
+    if (stream.status() == QTextStream::Ok) {
+        return wcroot;
+    } else {
+        return QString();
+    }
+}
+
 bool SvnCommands::updateToRevision(const QString& filePath, ulong revision)
 {
     QProcess process;
@@ -256,6 +285,38 @@ bool SvnCommands::revertToRevision(const QString& filePath, ulong revision)
     return true;
 }
 
+CommandResult SvnCommands::cleanup(const QString& dir, bool removeUnversioned, bool removeIgnored, bool includeExternals)
+{
+    QStringList arguments;
+    arguments << QStringLiteral("cleanup") << dir;
+    if (removeUnversioned) {
+        arguments << QStringLiteral("--remove-unversioned");
+    }
+    if (removeIgnored) {
+        arguments << QStringLiteral("--remove-ignored");
+    }
+    if (includeExternals) {
+        arguments << QStringLiteral("--include-externals");
+    }
+
+    QProcess process;
+    process.start(
+        QLatin1String("svn"),
+        arguments
+    );
+
+    CommandResult result;
+    if (!process.waitForFinished() || process.exitCode() != 0) {
+        result.success = false;
+    } else {
+        result.success = true;
+    }
+    result.stdout = process.readAllStandardOutput();
+    result.stderr = process.readAllStandardError();
+
+    return result;
+}
+
 bool SvnCommands::exportFile(const QUrl& path, ulong rev, QFileDevice *file)
 {
     if (file == nullptr || !path.isValid()) {
@@ -403,3 +464,24 @@ QSharedPointer< QVector<logEntry> > SvnCommands::getLog(const QString& filePath,
 
     return log;
 }
+
+bool SvnCommands::checkoutRepository(const QString& url, bool ignoreExternals, const QString& whereto)
+{
+    QStringList params;
+    params.append(QStringLiteral("checkout"));
+    params.append(url);
+    if (ignoreExternals) {
+        params.append(QStringLiteral("--ignore-externals"));
+    }
+    params.append(whereto);
+
+    QProcess process;
+    process.start(QLatin1String("svn"), params);
+
+    // Without timeout because it could be expensive time consuming operation.
+    if (!process.waitForFinished(-1) || process.exitCode() != 0) {
+        return false;
+    } else {
+        return true;
+    }
+}
