@@ -147,6 +147,10 @@ SvnLogDialog::~SvnLogDialog() = default;
 
 void SvnLogDialog::setCurrentRevision(ulong revision)
 {
+    if (m_log.isNull()) {
+        return;
+    }
+
     for (int i = 0; i < m_log->size(); ++i) {
         if (m_log->at(i).revision == revision) {
             QFont font;
@@ -157,6 +161,8 @@ void SvnLogDialog::setCurrentRevision(ulong revision)
             m_ui.tLog->item(i, columnDate)->setFont(font);
             m_ui.tLog->item(i, columnMessage)->setFont(font);
 
+            m_ui.tLog->selectRow(i);
+
             break;
         }
     }
@@ -165,6 +171,11 @@ void SvnLogDialog::setCurrentRevision(ulong revision)
 void SvnLogDialog::refreshLog()
 {
     m_log = SvnCommands::getLog(m_contextDir, m_logLength);
+
+    if (m_log.isNull()) {
+        return;
+    }
+
     m_ui.tLog->clearContents();
     m_ui.teMessage->clear();
     m_ui.lPaths->clear();
@@ -196,6 +207,9 @@ void SvnLogDialog::on_tLog_currentCellChanged(int currentRow, int currentColumn,
     if (currentRow < 0) {
         return;
     }
+    if (m_log.isNull()) {
+        return;
+    }
     if (m_log->size() < currentRow) {
         return;
     }
@@ -271,9 +285,8 @@ void SvnLogDialog::updateRepoToRevision()
         emit errorMessage(i18nc("@info:status", "SVN log: update to revision failed."));
     } else {
         emit operationCompletedMessage(i18nc("@info:status", "SVN log: update to revision %1 successful.", revision));
+        SvnLogDialog::refreshLog();
     }
-
-    SvnLogDialog::refreshLog();
 }
 
 void SvnLogDialog::revertRepoToRevision()
